import { strokeDashTypes } from "@react-financial-charts/core";
import { CurveFactoryLineOnly, CurveFactory } from "d3-shape";
import * as React from "react";
export interface LineSeriesProps {
    readonly canvasClip?: (context: CanvasRenderingContext2D, moreProps: any) => void;
    /**
     * Wether to connect the line between undefined data points.
     */
    readonly connectNulls?: boolean;
    /**
     * A factory for a curve generator for the line.
     */
    readonly curve?: CurveFactory | CurveFactoryLineOnly;
    /**
     * Function to decide if a data point has been defined.
     */
    readonly defined?: (d: number | undefined) => boolean;
    /**
     * Whether to highlight the line when within the `hoverTolerance`.
     */
    readonly highlightOnHover?: boolean;
    /**
     * Width to increase the line to on hover.
     */
    readonly hoverStrokeWidth?: number;
    /**
     * The distance between the cursor and the closest point in the line.
     */
    readonly hoverTolerance?: number;
    /**
     * Click handler.
     */
    readonly onClick?: (e: React.MouseEvent, moreProps: any) => void;
    /**
     * Double click handler.
     */
    readonly onDoubleClick?: (e: React.MouseEvent, moreProps: any) => void;
    /**
     * Hover handler.
     */
    readonly onHover?: (e: React.MouseEvent, moreProps: any) => void;
    /**
     * Unhover handler.
     */
    readonly onUnHover?: (e: React.MouseEvent, moreProps: any) => void;
    /**
     * Context menu handler.
     */
    readonly onContextMenu?: (e: React.MouseEvent, moreProps: any) => void;
    /**
     * Color, gradient, or pattern to use for the stroke.
     */
    readonly strokeStyle?: string;
    /**
     * Stroke dash.
     */
    readonly strokeDasharray?: strokeDashTypes;
    /**
     * Stroke width.
     */
    readonly strokeWidth?: number;
    /**
     * Selector for data to plot.
     */
    readonly yAccessor: (data: any) => number | undefined;
}
/**
 * `LineSeries` component.
 */
export declare class LineSeries extends React.Component<LineSeriesProps> {
    static defaultProps: {
        connectNulls: boolean;
        defined: (d: number | undefined) => boolean;
        hoverStrokeWidth: number;
        hoverTolerance: number;
        highlightOnHover: boolean;
        strokeDasharray: string;
        strokeStyle: string;
        strokeWidth: number;
    };
    render(): JSX.Element;
    private readonly drawOnCanvas;
    private readonly isHover;
}
