import * as React from "react";
import { isDefined, getAxisCanvas, GenericChartComponent } from "@react-financial-charts/core";
export class RenkoSeries extends React.Component {
    constructor() {
        super(...arguments);
        this.drawOnCanvas = (ctx, moreProps) => {
            const { xAccessor, xScale, chartConfig: { yScale }, plotData, } = moreProps;
            const renko = this.getRenko(plotData, xScale, xAccessor, yScale);
            renko.forEach((d) => {
                const { fillStyle, strokeStyle } = d;
                ctx.beginPath();
                if (strokeStyle !== undefined) {
                    ctx.strokeStyle = strokeStyle;
                }
                if (fillStyle !== undefined) {
                    ctx.fillStyle = fillStyle;
                }
                ctx.rect(d.x, d.y, d.width, d.height);
                ctx.closePath();
                ctx.fill();
            });
        };
        this.getRenko = (plotData, xScale, xAccessor, yScale) => {
            const { fill, stroke, yAccessor = RenkoSeries.defaultProps.yAccessor } = this.props;
            const width = xScale(xAccessor(plotData[plotData.length - 1])) - xScale(xAccessor(plotData[0]));
            const candleWidth = width / (plotData.length - 1);
            return plotData
                .filter((d) => isDefined(yAccessor(d).close))
                .map((d) => {
                const ohlc = yAccessor(d);
                const x = xScale(xAccessor(d)) - 0.5 * candleWidth;
                const y = yScale(Math.max(ohlc.open, ohlc.close));
                const height = Math.abs(yScale(ohlc.open) - yScale(ohlc.close));
                const fillStyle = d.fullyFormed ? (ohlc.open <= ohlc.close ? fill === null || fill === void 0 ? void 0 : fill.up : fill === null || fill === void 0 ? void 0 : fill.down) : fill === null || fill === void 0 ? void 0 : fill.partial;
                const strokeStyle = d.fullyFormed ? (ohlc.open <= ohlc.close ? stroke === null || stroke === void 0 ? void 0 : stroke.up : stroke === null || stroke === void 0 ? void 0 : stroke.down) : undefined;
                return {
                    fillStyle,
                    height,
                    strokeStyle,
                    width: candleWidth,
                    x,
                    y,
                };
            });
        };
    }
    render() {
        const { clip } = this.props;
        return (React.createElement(GenericChartComponent, { clip: clip, canvasDraw: this.drawOnCanvas, canvasToDraw: getAxisCanvas, drawOn: ["pan"] }));
    }
}
RenkoSeries.defaultProps = {
    clip: true,
    fill: {
        up: "#26a69a",
        down: "#ef5350",
        partial: "#4682B4",
    },
    stroke: {
        up: "none",
        down: "none",
    },
    yAccessor: (d) => ({ open: d.open, high: d.high, low: d.low, close: d.close }),
};
//# sourceMappingURL=RenkoSeries.js.map